package hash;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * 题目： 454. 四数相加 II ( fourSumCount)
 * 描述：给你四个整数数组 nums1、nums2、nums3 和 nums4 ，数组长度都是 n ，请你计算有多少个元组 (i, j, k, l) 能满足：
 *
 * 0 <= i, j, k, l < n
 * nums1[i] + nums2[j] + nums3[k] + nums4[l] == 0
 *
 示例 1：
 输入：nums1 = [1,2], nums2 = [-2,-1], nums3 = [-1,2], nums4 = [0,2]
 输出：2
 解释：
 两个元组如下：
 1. (0, 0, 0, 1) -> nums1[0] + nums2[0] + nums3[0] + nums4[1] = 1 + (-2) + (-1) + 2 = 0
 2. (1, 1, 0, 0) -> nums1[1] + nums2[1] + nums3[0] + nums4[0] = 2 + (-1) + (-1) + 0 = 0

 * 链接：https://leetcode.cn/problems/4sum-ii/
 */
//超时
public class FourSumCount {
    /**
     * 思路：把四数拆成两对 (A+B) 和 (C+D)
     * 先枚举 A 与 B 的所有和
     * 用 HashMap<Integer, Integer> 记录 “和 → 出现次数”。
     * 再枚举 C 与 D 的所有和
     * 对于每个和 s = c + d，要凑成 0 就需要 -(c+d)。
     * 看看 map 里有没有 -(c+d)，若有就把出现次数累加到结果里。
     * 时间复杂度
     * 两个 n² 循环 ⇒ 总 O(n²)。
     * 空间复杂度
     * 哈希表最多 n² 个键 ⇒ O(n²)。
     * @param nums1
     * @param nums2
     * @param nums3
     * @param nums4
     * @return
     */
    public int fourSumCount(int[] nums1, int[] nums2, int[] nums3, int[] nums4) {

        // 1) 统计 A+B 的所有组合和及其出现次数
        Map<Integer, Integer> abCount = new HashMap<>();
        for (int a : nums1) {
            for (int b : nums2) {
                int sum = a + b;
                abCount.put(sum, abCount.getOrDefault(sum, 0) + 1);
            }
        }

        // 2) 枚举 C+D，看 -(C+D) 是否出现在 map 里
        int ans = 0;
        for (int c : nums3) {
            for (int d : nums4) {
                int sum = c + d;
                // 需要 -(c+d) 才能凑成 0
                ans += abCount.getOrDefault(-sum, 0);
            }
        }
        return ans;
    }
}
